stochastic_simulate1_ctsmr <- function(object, nsim = 1L, newdata = NULL, firstorderinputinterpolation = FALSE, x0 = NULL, vx0 = NULL, nsub=10L, returnsubsampled=FALSE) 
{
    ## Inputs:
    ## object (fit from ctsmr)
    ## nsim = 1L (number of simulations)
    ## newdata = NULL (if not given the data from object is used)
    ## firstorderinputinterpolation = FALSE 
    ## nsub (Number of subsamples between timepoints)
    ## 
    ## Used for setting the initial value of the state vector in each simulation:
    ##   x0 = NULL (initial state value)
    ##   vx0 = NULL (initial state covariance)
    ##
    ctsmr:::ValidateResultObject(object)
    if (missing(newdata) || is.null(newdata)) {
        newdata <- object$data
        firstorderinputinterpolation <- attr(object$data, "FirstOrderInputInterpolation")
    }
    
    if(class(newdata) == "list"){
        if(length(newdata)>1){
            stop("supports only data in a data.frames or list of a single data.frame")
        }else{
            newdata <- newdata[[1]]
        }
    }

    m <- object$model

    ## Initial state
    x0prmnames <- paste0(m$states,"0")
    if (missing(x0) || is.null(x0)) {
        x0 <- object$xm[x0prmnames]
    }
    if (missing(vx0) || is.null(vx0)) {
        ## Covariance of the parameter estimates
        prmcov <- object$corr * (object$sd %*% t(object$sd))
        vx0 <- prmcov[x0prmnames,x0prmnames, drop=FALSE]
    }

    ## covariance is used when calling predict iterating for each step
    covariance <- TRUE
    outputsize <- 2L * (m$N + m$S) + ifelse(covariance, m$N * 
                                                        (m$N - 1L)/2L + m$S * (m$S - 1L)/2L, 0L)
    ## Subsampling
    data <- subsample(newdata, nsub, firstorderinputinterpolation)
    ## In order to, when doing 1-step ahead predictions, not get an update, then set the output to NA
    data[ ,m$outputs] <- NA
    
    ## data.frames to keep the simualted results in later
    statesMat <- matrix(NA, nrow=nrow(data), ncol=length(m$states))
    colnames(statesMat) <- names(x0)
    statesMat[1, ] <- x0
    ##
    outMat <- matrix(NA, nrow=nrow(data), ncol=length(m$outputs))
    colnames(outMat) <- m$outputs
    
    ## In each step the initial state covariance is set as low as possible (cannot be zero), such that it almost doesn't affect the covariance of the prediction
    vx0onestep <- diag(.Machine$double.xmin, m$N)
    
    ## Go
    Lsim <- lapply(1:nsim, function(ii){
        ## Init the state
        ## For now sample with state init estimates only
        statesMat[m$N, ] <- mvrnorm(1, x0, vx0)
        ## Run for all the subsamples
        for(i in 1:(nrow(data)-1)){
            ## Do the one-step prediction
            tmp <- ctsmr:::predict.ctsmr(object, covariance=TRUE, newdata=data[i:(i+1), ], firstorder=firstorderinputinterpolation, x0=statesMat[i, ], vx0=vx0onestep)
            ## New states with system noise
            statesMat[i+1, ] <- unlist(tmp$state$pred[2, ]) + mvrnorm(1, numeric(m$N), tmp$state$var[ , ,2])
            ## ## Test compare:
            ## tmp$state$var[ , ,2]
            ## sigma <- eval(m$sys.eqs[[1]]$parsed[[2]], as.list(c(object$xm,data[i+1, ])))
            ## sigma^2 * (data$t[i+1] - data$t[i])
            ## predict of previous step output which has system noise added
            outMat[i, ] <- tmp$output$pred[1, ] + mvrnorm(1, numeric(length(m$outputs)), tmp$output$var[ , ,1])
            ## ## Test compare:
            ## tmp$output$var[ , ,1]
            ## eval(m$variance[[1]]$var, as.list(c(object$xm,data[i, ])))
        }
        ## Name states correctly
        colnames(statesMat) <- m$states
        ## Get the last output value out (hacked by calling predict for reversed last data points)
        i <- nrow(data)
        tmpdata <- data[i:(i-1), ]
        tmpdata$t <- c(tmpdata$t[2], tmpdata$t[1])
        tmp <- ctsmr:::predict.ctsmr(object, covariance=TRUE, newdata=tmpdata, firstorder=firstorderinputinterpolation, x0=statesMat[i, ], vx0=vx0onestep)
        outMat[i, ] <- tmp$output$pred[1, ] + mvrnorm(1, numeric(length(m$outputs)), tmp$output$var[ , ,1])
        ## 
        if( returnsubsampled ){
            x <- as.data.frame(statesMat)
            y <- as.data.frame(outMat)
            time <- data$t
        }else{
            ## Get out only the values at t
            i <- (0:(nrow(newdata)-1))*(nsub+1) + 1
            ## check: all(data$t[i] == newdata$t)
            x <- as.data.frame(statesMat[i, , drop=FALSE])
            y <- as.data.frame(outMat[i, , drop=FALSE])
            time <- data$t[i]
        }
        ## Return
        list(t = time, state = x, output = y)
    })
    ## Return
    return(Lsim)
}
